#! /usr/bin/perl

use strict;
use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser);
use Math::Trig;

my $i;
my $alpha = 0.0;
my $PI = 3.1415927;
my $x = 0.0;
my $y = 0.0;
my $z = 0.0;
my $factor = 1.00;

my $drill_d = 2.0;
my $drill_r = $drill_d/2.0;
my $chain_link_w = 10;
my $tooth_no = 10;
my $tooth_width_1 = 0.25;
my $tooth_width_2 = 0.0;
my $gap = 0.005;
my $radius_outer = 20 + $drill_r;
my $radius_screws = 24;
my $Screws_r = (3.0-$drill_d)/2.0;
my $servo_horn_r = 27/2.0;
my $servo_hole_r = 5.0;

my $flap_l = 40;
my $flap_w = 10;

my $bar_gap = 0.75;
my $bar_gap_2 = 0.21;
my $bar_gap_3 = 4;

my $holder_w = 75;
my $holder_l = 60;

my $cut_edge = 5;
my $cable_w = 20;
my $cable_l = 30;

my $mount_holes_l = 40;
my $mount_holes_w = 60;

my $screw_hole_r = (3.00-$drill_d)/2.0;

my $x_temp = 0.0;
my $y_temp = 0.0;

my $x_distance = $holder_w/2;
my $y_distance = $holder_l/2;

my $cut_gap = 4.5;   #Width of cut gaps, 3.0mm drill


my $step_angle = 2.0 * $PI/$tooth_no;



my $feed_rate = 1.000;
my $feed_rate_material = 1.500;
my $feed_rate_empty = 2.500;
my $Z_up = 3.000;
my $Z_down = -1.000;
my $Z_down_1 = -1.500;
my $Z_down_2 = -2.000;
my $Z_down_3 = -4.500;
my $z_step = 4.5;
my $z_step_temp = $z_step;

open(FILEOUT,'> camera-board-holder.gcode') || die "Can not open " . "camera-board-holder.gcode!\n";


  print FILEOUT "M3\n";
  print FILEOUT "M8\n";


  $z = $Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


  #mount holes
  &screwHoles(-$mount_holes_w/2.0, -$mount_holes_l/2.0);
  &screwHoles($mount_holes_w/2.0, -$mount_holes_l/2.0);
  &screwHoles($mount_holes_w/2.0, $mount_holes_l/2.0);
  &screwHoles(-$mount_holes_w/2.0, $mount_holes_l/2.0);




  #Outer cut line
  $x = -$holder_w/2 - $drill_r;
  $y = -$holder_l/2 - $drill_r + $cut_edge;
  $x = $x * $factor + $x_distance;
  $y = $y * $factor + $y_distance;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";



  for($Z_down = -$z_step; $Z_down >= $Z_down_3; $Z_down -= $z_step){
    $z = $Z_down;
    $feed_rate = $feed_rate_material;


    $x = -$holder_w/2 - $drill_r;
    $y = -$holder_l/2 - $drill_r + $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = -$holder_w/2 - $drill_r + $cut_edge;
    $y = -$holder_l/2 - $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


    #Cut gap
    $x = -$holder_w/3 - $drill_r;
    $y = -$holder_l/2 - $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    if($Z_down < $Z_down_2){
      $z = $Z_down_2;

      $x = -$holder_w/3 - $drill_r;
      $y = -$holder_l/2 - $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = -$holder_w/3 - $drill_r + $cut_gap;
      $y = -$holder_l/2 - $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    }

    $z = $Z_down;

    $x = -$holder_w/3 - $drill_r + $cut_gap;
    $y = -$holder_l/2 - $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


    $x = $holder_w/3 + $drill_r;
    $y = -$holder_l/2 - $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    if($Z_down < $Z_down_2){
      $z = $Z_down_2;

      $x = $holder_w/3 + $drill_r;
      $y = -$holder_l/2 - $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = $holder_w/3 + $drill_r + $cut_gap;
      $y = -$holder_l/2 - $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    }

    $z = $Z_down;

    $x = $holder_w/3 + $drill_r + $cut_gap;
    $y = -$holder_l/2 - $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";



    $x = $holder_w/2 + $drill_r - $cut_edge;
    $y = -$holder_l/2 - $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = $holder_w/2 + $drill_r;
    $y = -$holder_l/2 - $drill_r + $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


    $x = $holder_w/2 + $drill_r;
    $y = $holder_l/2 + $drill_r - $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


    $x = $holder_w/2 + $drill_r - $cut_edge;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";



    #Cut gap
    $x = $holder_w/3 + $drill_r + $cut_gap;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    if($Z_down < $Z_down_2){
      $z = $Z_down_2;

      $x = $holder_w/3 + $drill_r + $cut_gap;
      $y = $holder_l/2 + $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = $holder_w/3 + $drill_r;
      $y = $holder_l/2 + $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    }

    $z = $Z_down;

    $x = $holder_w/3 + $drill_r;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";





    $x = $cable_w/2 - $drill_r + $cut_edge;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


    $x = $cable_w/2 - $drill_r;
    $y = $holder_l/2 + $drill_r - $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = $cable_w/2 - $drill_r;
    $y = $holder_l/2 - $cable_l + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = -$cable_w/2 + $drill_r;
    $y = $holder_l/2 - $cable_l + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = -$cable_w/2 + $drill_r;
    $y = $holder_l/2 + $drill_r - $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = -$cable_w/2 + $drill_r - $cut_edge;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";



    #Cut gap
    $x = -$holder_w/3 - $drill_r + $cut_gap;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    if($Z_down < $Z_down_2){
      $z = $Z_down_2;

      $x = -$holder_w/3 - $drill_r + $cut_gap;
      $y = $holder_l/2 + $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = -$holder_w/3 - $drill_r;
      $y = $holder_l/2 + $drill_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    }

    $z = $Z_down;

    $x = -$holder_w/3 - $drill_r;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";




    $x = -$holder_w/2 - $drill_r + $cut_edge;
    $y = $holder_l/2 + $drill_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = -$holder_w/2 - $drill_r;
    $y = $holder_l/2 + $drill_r - $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = -$holder_w/2 - $drill_r;
    $y = -$holder_l/2 - $drill_r + $cut_edge;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


    if($Z_down - $z_step < $Z_down_3 && $Z_down !=  $Z_down_3){
      $Z_down = $Z_down_3 + $z_step;
    }
  }

  $z = $Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  print FILEOUT "M5\n";
  print FILEOUT "M9\n";


  $x=0.0;
  $y=0.0;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


close(FILEOUT);


#++++++++++++++++++++++ Screw holes ++++++++++++++++++++++++++++++++++++
sub screwHoles(){

  my $nut_r = 5.3/2.0 - $drill_r;
  my $r = $nut_r;
  my $nut_depth = -1.9;

  if($_[2] == 0){
    $r = $screw_hole_r;
  }

  $x = $_[0] + sin(0.0) * ($r)/2.0;
  $y = $_[1] + cos(0.0) * ($r)/2.0;
  $x = $x * $factor + $x_distance;
  $y = $y * $factor + $y_distance;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  for($Z_down = -$z_step; $Z_down >= $Z_down_3; $Z_down -= $z_step){
    if($Z_down < $nut_depth && $r == $nut_r){
      $Z_down = $nut_depth;
    }
    $z = $Z_down;
    $feed_rate = $feed_rate_material;
    for($alpha = 0.0; $alpha <= $PI * 2.0; $alpha += 0.01){
      $x = $_[0] + sin($alpha) * $r;
      $y = $_[1] + cos($alpha) * $r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    }

    if($Z_down == $nut_depth){
      $r = $screw_hole_r;
      $x = $_[0] + sin(0.0) * $r;
      $y = $_[1] + cos(0.0) * $r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";
    }

    if($Z_down - $z_step < $Z_down_3 && $Z_down !=  $Z_down_3){
      $Z_down = $Z_down_3 + $z_step;
    }
  }

  $z=$Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

}
