#! /usr/bin/perl

use strict;
use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser);
use Math::Trig;

my $i;
my $alpha = 0.0;
my $PI = 3.1415927;
my $x = 0.0;
my $y = 0.0;
my $z = 0.0;
my $factor = 1.00;

my $drill_d = 2.0;
my $drill_r = $drill_d/2.0;
my $chain_link_w = 10;
my $tooth_no = 10;
my $tooth_width_1 = 0.25;
my $tooth_width_2 = 0.0;
my $gap = 0.005;
my $radius_outer = 44;
my $radius_inner = 41;
my $radius_bend = 36;
my $radius_screws = 24;
my $Screws_r = (3.0-$drill_d)/2.0;
my $servo_horn_r = 27/2.0;
my $servo_hole_r = 5.0;

my $bar_gap = 0.55;

my $x_distance = $radius_outer + $drill_r;
my $y_distance = $radius_outer + $drill_r;

my $drill_a = asin($drill_r/$radius_inner);

my $step_angle = 2.0 * $PI/$tooth_no;

$tooth_width_2 = $step_angle - asin(($chain_link_w/2)/$radius_inner)*2.0;


my $feed_rate = 1.000;
my $feed_rate_material = 1.500;
my $feed_rate_empty = 2.500;
my $Z_up = 3.000;
my $Z_down = -1.000;
my $Z_down_1 = -1.500;
my $Z_down_2 = -2.000;
my $Z_down_3 = -4.500;
my $z_step = 2.25;
my $z_step_temp = $z_step;

open(FILEOUT,'> DriveWheel.gcode') || die "Can not open " . "DriveWheel.gcode!\n";


  print FILEOUT "M3\n";
  print FILEOUT "M8\n";


  $z = $Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  #Bend

  $x = sin(0.0) * ($radius_bend);
  $y = cos(0.0) * ($radius_bend);
  $x = $x * $factor + $x_distance;
  $y = $y * $factor + $y_distance;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  if($z_step_temp > -$Z_down_1){
    $z_step_temp = -$Z_down_1;
  }
  for($Z_down = -$z_step_temp; $Z_down >= $Z_down_1; $Z_down -= $z_step_temp){
    $z = $Z_down;
    $feed_rate = $feed_rate_material;
    for($alpha = 0.0; $alpha <= 2.0 * $PI + $step_angle; $alpha += $step_angle){
      $x = sin($alpha) * ($radius_bend);
      $y = cos($alpha) * ($radius_bend);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    }
    if($Z_down - $z_step < $Z_down_1 && $Z_down !=  $Z_down_1){
      $Z_down = $Z_down_1 + $z_step;
    }
  }


  #Screw holes
  for($alpha = 0.0; $alpha < 2.0 * $PI; $alpha += 2.0 * $step_angle){
    my $xr = sin($alpha) * ($radius_screws);
    my $yr = cos($alpha) * ($radius_screws);
    my $beta = 0.0;

    $z = $Z_up;
    $feed_rate = $feed_rate_empty;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";
    $x = $xr + sin($beta) * $Screws_r;
    $y = $yr + cos($beta) * $Screws_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    for($Z_down = -$z_step; $Z_down >= $Z_down_3; $Z_down -= $z_step){
      $z = $Z_down;
      $feed_rate = $feed_rate_material;
      for($beta = 0.0; $beta <= 2.0 * $PI; $beta += 0.01){
        $x = $xr + sin($beta) * $Screws_r;
        $y = $yr + cos($beta) * $Screws_r;
        $x = $x * $factor + $x_distance;
        $y = $y * $factor + $y_distance;
        print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      }
      if($Z_down - $z_step < $Z_down_3 && $Z_down !=  $Z_down_3){
        $Z_down = $Z_down_3 + $z_step;
      }
    }
  }


  #Servo horn holes
  for($alpha = 0.0; $alpha < 2.0 * $PI; $alpha += $PI / 2.0){
    my $xr = sin($alpha) * ($servo_horn_r);
    my $yr = cos($alpha) * ($servo_horn_r);
    my $beta = 0.0;

    $z = $Z_up;
    $feed_rate = $feed_rate_empty;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    $x = $xr + sin($beta) * $Screws_r;
    $y = $yr + cos($beta) * $Screws_r;
    $x = $x * $factor + $x_distance;
    $y = $y * $factor + $y_distance;
    print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

    for($Z_down = -$z_step; $Z_down >= $Z_down_3; $Z_down -= $z_step){
      $z = $Z_down;
      $feed_rate = $feed_rate_material;
      for($beta = 0.0; $beta <= 2.0 * $PI; $beta += 0.01){
        $x = $xr + sin($beta) * $Screws_r;
        $y = $yr + cos($beta) * $Screws_r;
        $x = $x * $factor + $x_distance;
        $y = $y * $factor + $y_distance;
        print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";
      }
      if($Z_down - $z_step < $Z_down_3 && $Z_down !=  $Z_down_3){
        $Z_down = $Z_down_3 + $z_step;
      }
    }
  }

  $z = $Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  $x = sin(0.0) * $servo_hole_r;
  $y = cos(0.0) * $servo_hole_r;
  $x = $x * $factor + $x_distance;
  $y = $y * $factor + $y_distance;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  for($Z_down = -$z_step; $Z_down >= $Z_down_3; $Z_down -= $z_step){
    $z = $Z_down;
    $feed_rate = $feed_rate_material;
    for($alpha = 0.0; $alpha <= 2.0 * $PI; $alpha += 0.01){
      if($Z_down < $Z_down_2 && ($alpha < $bar_gap || $alpha > $PI*4/3 && $alpha < $PI*4/3 + $bar_gap || $alpha > $PI*2/3 && $alpha < $PI*2/3 + $bar_gap)){
        $z = $Z_down_2;
      }
      else{
        $z = $Z_down;
      }
      $x = sin($alpha) * $servo_hole_r;
      $y = cos($alpha) * $servo_hole_r;
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";
    }
    if($Z_down - $z_step < $Z_down_3 && $Z_down !=  $Z_down_3){
      $Z_down = $Z_down_3 + $z_step;
    }
  }

  $z = $Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  #Outer cut line
  $x = sin(0.0) * ($radius_bend - $drill_r);
  $y = cos(0.0) * ($radius_bend - $drill_r);
  $x = $x * $factor + $x_distance;
  $y = $y * $factor + $y_distance;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


  for($Z_down = -$z_step; $Z_down >= $Z_down_3; $Z_down -= $z_step){
    $z = $Z_down;
    $feed_rate = $feed_rate_material;
    $i = 0;
    for($alpha = 0.0; $alpha < 2.0 * $PI; $alpha += $step_angle){
      $x = sin($alpha) * ($radius_bend - $drill_r);
      $y = cos($alpha) * ($radius_bend - $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


      $x = sin($alpha + $gap/2.0) * ($radius_outer + $drill_r);
      $y = cos($alpha + $gap/2.0) * ($radius_outer + $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      if($Z_down < $Z_down_2 && ($i==0 || $i== 4 || $i==7)){
        $z = $Z_down_2;
        print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";
      }
      $x = sin($alpha + $tooth_width_1/2.0) * ($radius_outer + $drill_r);
      $y = cos($alpha + $tooth_width_1/2.0) * ($radius_outer + $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      if($z > $Z_down){
        $z = $Z_down;
        print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";
      }

      $x = sin($alpha + $tooth_width_2/2.0) * ($radius_inner + $drill_r);
      $y = cos($alpha + $tooth_width_2/2.0) * ($radius_inner + $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = sin($alpha + $step_angle - $tooth_width_2/2.0) * ($radius_inner + $drill_r);
      $y = cos($alpha + $step_angle - $tooth_width_2/2.0) * ($radius_inner + $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = sin($alpha + $step_angle - $tooth_width_1/2.0) * ($radius_outer + $drill_r);
      $y = cos($alpha + $step_angle - $tooth_width_1/2.0) * ($radius_outer + $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $x = sin($alpha + $step_angle - $gap/2.0) * ($radius_outer + $drill_r);
      $y = cos($alpha + $step_angle - $gap/2.0) * ($radius_outer + $drill_r);
      $x = $x * $factor + $x_distance;
      $y = $y * $factor + $y_distance;
      print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

      $i++;
    }

    if($Z_down - $z_step < $Z_down_3 && $Z_down !=  $Z_down_3){
      $Z_down = $Z_down_3 + $z_step;
    }
  }
  $z = $Z_up;
  $feed_rate = $feed_rate_empty;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";

  print FILEOUT "M5\n";
  print FILEOUT "M9\n";


  $x=0.0;
  $y=0.0;
  print FILEOUT "X" . sprintf("%.3f", $x)  . " Y" . sprintf("%.3f", $y)  .  " Z" . sprintf("%.3f", $z)  . " F" . sprintf("%.3f", $feed_rate)  . "\n";


close(FILEOUT);
