//compile with: gcc R12C1.c -lmraa -o R12C1

#include "mraa.h"
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#define SERVO_R        5
#define SERVO_L        6
#define SERVO_CAMERA   3

#define LIGHT_1       10

#define FLASH_LIGHT   13

#define ADC_BATTERY    2
//ADC 457 = 7.45V
//ADC 504 = 8.15V
#define BATTERY_EMPTY 435.0
//#define BATTERY_EMPTY 455.0

#define SERVO_R_FORWARD   1000
#define SERVO_R_BACKWARD  2000

#define SERVO_L_FORWARD   2000
#define SERVO_L_BACKWARD  1000

#define SERVO_CAMERA_UP         900
#define SERVO_CAMERA_STRAIGHT  1400
#define SERVO_CAMERA_DOWN      1700




int run = 1;

int main (int argc, char **argv){
  int i;
  FILE *commandsFile;
  char commandsLine[1000];
  char a = 0;
  long cameraAngle = 0;
  long driveDistance = 0;
  char convertChar[10];
  float batteryStatus = 0.0;
  time_t startTime = time(NULL);

  //Initialize servo pins
  mraa_pwm_context pwmSl = mraa_pwm_init(SERVO_L);
  mraa_pwm_enable(pwmSl,0 );
  mraa_pwm_period_ms(pwmSl, 20);
  mraa_pwm_enable(pwmSl, 1);

  mraa_pwm_context pwmSr = mraa_pwm_init(SERVO_R);
  mraa_pwm_enable(pwmSr, 0);
  mraa_pwm_period_ms(pwmSr, 20);
  mraa_pwm_enable(pwmSr, 1);

  mraa_pwm_context pwmSc = mraa_pwm_init(SERVO_CAMERA);
  mraa_pwm_enable(pwmSc, 0);
  mraa_pwm_period_ms(pwmSc, 20);
  mraa_pwm_enable(pwmSc, 1);

  //initialize light pin
  mraa_pwm_context pwmLight1 = mraa_pwm_init(LIGHT_1);
  mraa_pwm_enable(pwmLight1, 0);
  mraa_pwm_period_ms(pwmLight1, 20);
  mraa_pwm_enable(pwmLight1, 1);


  //initialize flash light pin
  mraa_gpio_context flashLight = mraa_gpio_init(FLASH_LIGHT);
  if (flashLight == NULL) {
    fprintf(stderr, "Are you sure that pin %d you requested is valid on your platform?", FLASH_LIGHT);
    exit(1);
  }
  mraa_gpio_dir(flashLight, MRAA_GPIO_OUT);

  //Initialize ADC battery pin
  mraa_aio_context adcBattery = mraa_aio_init(ADC_BATTERY);

  system("rm Commands.txt");

  while(run){

    if(access("Commands.txt", W_OK) != -1){
      //sleep(1);
      if (!(commandsFile = fopen("Commands.txt", "r"))){
        printf("Can't open Commands.txt!\n");
        run = 0;
      }
      else{
        a = 0;
        i = 0;
        commandsLine[0] = '\0';
        while((!(feof(commandsFile))) && a != 13 && a != 10 && i < 1000){
          fread(&a, 1, 1, commandsFile);
          commandsLine[i] = a;
          commandsLine[i+1] = '\0';
          i++;
          //printf("'%c'", a);
        }
        printf("commandsLine='%s'\n", commandsLine);

        if(strstr(commandsLine, "ShutDown")!=NULL){
          system("rm Commands.txt");
          mraa_gpio_write(flashLight, 1);
          for(i=0; i<10; i++){
            mraa_pwm_pulsewidth_us (pwmLight1, 20000);
            usleep(500000);
            mraa_pwm_pulsewidth_us (pwmLight1, 0);
            usleep(500000);
          }
          mraa_gpio_write(flashLight, 0);
          printf("Shutting down IOT2020!\n");
          system("init 0");
        }

        mraa_gpio_write(flashLight, 1);
        usleep(500000);


        convertChar[0]=commandsLine[11];
        convertChar[1]=commandsLine[12];
        convertChar[2]='\0';
        cameraAngle = strtol(convertChar, NULL, 10);

        if(cameraAngle == 6){//Camera servo up
          printf("Camera to position %ld\n", cameraAngle);
          mraa_pwm_pulsewidth_us (pwmSc, SERVO_CAMERA_UP);
          usleep(1500000);//Time to settle camera servo
          mraa_pwm_pulsewidth_us (pwmSc, 0);
        }
        if(cameraAngle == 5){//Camera servo straight
          printf("Camera to position %ld\n", cameraAngle);
          mraa_pwm_pulsewidth_us (pwmSc, SERVO_CAMERA_STRAIGHT);
          usleep(1500000);//Time to settle camera servo
          mraa_pwm_pulsewidth_us (pwmSc, 0);
        }
        if(cameraAngle == 4){//Camera servo down
          printf("Camera to position %ld\n", cameraAngle);
          mraa_pwm_pulsewidth_us (pwmSc, SERVO_CAMERA_DOWN);
          usleep(1500000);//Time to settle camera servo
          mraa_pwm_pulsewidth_us (pwmSc, 0);
        }


        convertChar[0]=commandsLine[7];
        convertChar[1]=commandsLine[8];
        convertChar[2]='\0';
        driveDistance = strtol(convertChar, NULL, 10);
        driveDistance *= 500000;
        if(commandsLine[6] == '0'){//Drive forward
          printf("Driving forward for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_FORWARD);
          mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_FORWARD);
          usleep(driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }

        if(commandsLine[6] == '3'){//Drive backward
          printf("Driving backward for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_BACKWARD);
          mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_BACKWARD);
          usleep(driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }

        if(commandsLine[6] == '6'){//Turn left
          printf("Turning left for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_BACKWARD);
          mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_FORWARD);
          usleep(driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }

        if(commandsLine[6] == '7'){//Turn right
          printf("Turning right for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_FORWARD);
          mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_BACKWARD);
          usleep(driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }


        if(commandsLine[6] == '1'){//Front left
          printf("Front left for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_FORWARD);
          for(i=0; i<driveDistance/240000; i++){
            mraa_pwm_pulsewidth_us (pwmSl, 0);
            usleep(160000);
            mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_FORWARD);
            usleep(80000);
          }
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }

        if(commandsLine[6] == '2'){//Front right
          printf("Front right for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_FORWARD);
          for(i=0; i<driveDistance/240000; i++){
            mraa_pwm_pulsewidth_us (pwmSr, 0);
            usleep(160000);
            mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_FORWARD);
            usleep(80000);
          }
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }

        if(commandsLine[6] == '4'){//Back left
          printf("Back left for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_BACKWARD);
          for(i=0; i<driveDistance/240000; i++){
            mraa_pwm_pulsewidth_us (pwmSl, 0);
            usleep(160000);
            mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_BACKWARD);
            usleep(80000);
          }
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }

        if(commandsLine[6] == '5'){//Back right
          printf("Back right for %ldus\n", driveDistance);
          mraa_pwm_pulsewidth_us (pwmSl, SERVO_L_BACKWARD);
          for(i=0; i<driveDistance/240000; i++){
            mraa_pwm_pulsewidth_us (pwmSr, 0);
            usleep(160000);
            mraa_pwm_pulsewidth_us (pwmSr, SERVO_R_BACKWARD);
            usleep(80000);
          }
          mraa_pwm_pulsewidth_us (pwmSl, 0);
          mraa_pwm_pulsewidth_us (pwmSr, 0);
        }


        //headlight control
        if(commandsLine[13] == '0'){
          printf("Set headlight to 0\n");
          mraa_pwm_pulsewidth_us (pwmLight1, 0);
        }
        if(commandsLine[13] == '1'){
          printf("Set headlight to 1\n");
          mraa_pwm_pulsewidth_us (pwmLight1, 5000);
        }
        if(commandsLine[13] == '2'){
          printf("Set headlight to 2\n");
          mraa_pwm_pulsewidth_us (pwmLight1, 10000);
        }
        if(commandsLine[13] == '3'){
          printf("Set headlight to 3\n");
          mraa_pwm_pulsewidth_us (pwmLight1, 15000);
        }
        if(commandsLine[13] == '4'){
          printf("Set headlight to 4\n");
          mraa_pwm_pulsewidth_us (pwmLight1, 20000);
        }

      }
      system("rm Commands.txt");
      system("cp IndicationPicture.bmp R12_CommandFree.bmp");

      mraa_gpio_write(flashLight, 0);
    }

    if(time(NULL) - startTime > 30){
      //Print battery status
      batteryStatus = mraa_aio_read(adcBattery);
      printf("batteryStatus=%3.5lf\n", batteryStatus);
      sprintf(commandsLine, "echo \"Battery = % 5.0lf/% 6.0lf\n\" > Battery.txt", batteryStatus, BATTERY_EMPTY);
      system(commandsLine);
      if (batteryStatus <= BATTERY_EMPTY){
        for(i=0; i<50; i++){
          mraa_pwm_pulsewidth_us (pwmLight1, 20000);
          usleep(500000);
          mraa_pwm_pulsewidth_us (pwmLight1, 0);
          usleep(500000);
        }
        mraa_gpio_write(flashLight, 0);
        printf("Battery low! Shutting down...\n");
        system("init 0");
        sleep(100);
      }
      startTime = time(NULL);
    }

   }//while(run)



  return MRAA_SUCCESS;
}
